import os
import json

import click
import pygments

from .core import Combine
from .dev import Watcher, Server
from .exceptions import BuildError


@click.group()
@click.pass_context
def cli(ctx):
    pass


@cli.command()
@click.option("--env", default="production")
@click.option("--var", multiple=True, default=[])
@click.pass_context
def build(ctx, env, var):
    variables = dict(x.split("=") for x in var)
    config_path = os.path.abspath("combine.yml")
    combine = Combine(config_path=config_path, env=env, variables=variables)

    click.secho("Building site", fg="cyan")
    try:
        combine.build()
    except BuildError:
        click.secho("Build error (see above)", fg="red")
        exit(1)

    return combine


@cli.command()
@click.option("--port", type=int, default=8000)
@click.pass_context
def work(ctx, port):
    combine = ctx.invoke(
        build, env="development", var=[f"base_url=http://127.0.0.1:{port}"]
    )

    click.secho("Watching for file changes...", fg="green")

    server = Server(combine.output_path, port)
    watcher = Watcher(".", combine=combine)
    watcher.watch(server.serve)


@cli.group()
@click.pass_context
def utils(ctx):
    pass


@utils.command()
@click.option("--style", type=str, default="default")
@click.pass_context
def highlight_info(ctx, style):
    """Outputs the CSS which can be customized for highlighted code"""
    click.secho("The following styles are available to choose from:", fg="green")
    click.echo(list(pygments.styles.get_all_styles()))
    click.echo()
    click.secho(
        f'The following CSS for the "{style}" style can be customized:', fg="green"
    )
    click.echo(pygments.formatters.HtmlFormatter(style=style).get_style_defs())


if __name__ == "__main__":
    cli()



class MissingVariableError(Exception):
    def __init__(self, name):
        self.name = name
        self.message = f'The required variable "{self.name}" is missing'
        super().__init__(self.message)


class ReservedVariableError(Exception):
    def __init__(self, name):
        self.name = name
        self.message = (
            f'The variable"{self.name}" is reserved and should only be set by combine'
        )
        super().__init__(self.message)



from jinja2 import nodes
from jinja2.ext import Extension
from jinja2 import Markup


class IncludeRawExtension(Extension):
    tags = {"include_raw"}

    def parse(self, parser):
        lineno = parser.stream.expect("name:include_raw").lineno
        filename = nodes.Const(parser.parse_expression().value)
        result = self.call_method("_render", [filename], lineno=lineno)
        return nodes.Output([result], lineno=lineno)

    def _render(self, filename):
        return Markup(self.environment.loader.get_source(self.environment, filename)[0])



import os

import frontmatter

from .html import HTMLFile
from .utils import create_parent_directory


class MarkdownFile(HTMLFile):
    def get_path_for_output(self):
        if self.name_without_extension.endswith(".keep"):
            # remove .keep.md from the end and replace with .html
            return super().get_path_for_output()[:-8] + ".html"

        if self.name_without_extension == "index":
            return super().get_path_for_output()[:-3] + ".html"

        return os.path.join(*self.root_parts, "index.html")

    def render_to_output(self, output_path, *args, **kwargs):
        post = frontmatter.load(self.path)

        variables = post.metadata
        variables["url"] = self._get_url()
        variables["content"] = post.content

        # TODO can post.content be jinja rendered to use variables?

        # an optional way to override
        template_path = variables.get("template", "markdown.template.html")
        template = kwargs["jinja_environment"].get_template(template_path)

        target_path = os.path.join(output_path, self.output_relative_path)
        create_parent_directory(target_path)

        with open(target_path, "w+") as f:
            f.write(template.render(**variables))



from .code import CodeHighlightExtension
from .markdown import MarkdownExtension
from .include_raw import IncludeRawExtension
from .urls import absolute_url


default_extensions = [CodeHighlightExtension, MarkdownExtension, IncludeRawExtension]
default_filters = {
    "absolute_url": absolute_url,
}



import os
import traceback

from .html import HTMLFile
from .utils import create_parent_directory


class ErrorFile(HTMLFile):
    def __init__(self, *args, **kwargs):
        self.error = kwargs.pop("error")
        super().__init__(*args, **kwargs)

    def render_to_output(self, output_path, *args, **kwargs):
        target_path = os.path.join(output_path, self.output_relative_path)
        create_parent_directory(target_path)

        template = kwargs["jinja_environment"].get_template("error.template.html")

        with open(target_path, "w+") as f:
            context_lines = "\n".join(
                self.error.source.splitlines()[
                    max(self.error.lineno - 3, 0) : self.error.lineno + 3
                ]
            )
            f.write(
                template.render(
                    error=self.error,
                    relative_path=self.content_relative_path,
                    context_lines=context_lines,
                    excinfo=traceback.format_exc(),
                )
            )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['combine', 'combine.files', 'combine.jinja']

package_data = \
{'': ['*'], 'combine': ['base_content/*']}

install_requires = \
['Pygments>=2.6.1,<3.0.0',
 'beautifulsoup4>=4.9.1,<5.0.0',
 'click>=7.1.2,<8.0.0',
 'jinja2>=2.11.2,<3.0.0',
 'markdown>=3.2.2,<4.0.0',
 'python-frontmatter>=0.5.0,<0.6.0',
 'pyyaml>=5.3.1,<6.0.0',
 'watchdog>=0.10.3,<0.11.0']

entry_points = \
{'console_scripts': ['combine = combine.cli:cli']}

setup_kwargs = {
    'name': 'combine',
    'version': '0.2.0',
    'description': 'A straightforward static site builder.',
    'long_description': '# combine\n',
    'author': 'Dropseed',
    'author_email': 'python@dropseed.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://combine.dropseed.io',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import os
import datetime

import yaml


class Config:
    def __init__(self, path):
        self.path = path
        self.data = {}

        if os.path.exists(self.path):
            self.data = yaml.safe_load(open(self.path, "r"))
            if self.data is None:
                # allow an empty file to start with
                self.data = {}

    @property
    def output_path(self):
        return os.path.abspath(self.data.get("output_path", "output"))

    @property
    def content_paths(self):
        if "content_paths" in self.data:
            paths = self.data["content_paths"]
        else:
            paths = ["content"]
            if os.path.exists(os.path.join("theme", "content")):
                paths.append(os.path.join("theme", "content"))

        # add the built-in content from combine itself
        paths.append(os.path.join(os.path.dirname(__file__), "base_content"))

        return [os.path.abspath(x) for x in paths]

    @property
    def variables(self):
        variables = self.default_variables

        for name, data in self.data.get("variables", {}).items():
            if isinstance(data, dict):
                if "default" in data:
                    variables[name] = data["default"]

                if "from_env" in data and data["from_env"] in os.environ:
                    variables[name] = os.environ[data["from_env"]]
            else:
                variables[name] = data

        return variables

    @property
    def default_variables(self):
        return {"now": datetime.datetime.now}  # as a function

    @property
    def steps(self):
        return self.data.get("steps", [])



import os

from .core import File
from .ignored import IgnoredFile
from .template import TemplateFile
from .html import HTMLFile
from .markdown import MarkdownFile
from .redirect import RedirectFile
from .error import ErrorFile


def file_class_for_path(path):
    base, ext = os.path.splitext(path)

    if os.path.basename(path).startswith("_") or os.path.basename(path).startswith("."):
        return IgnoredFile

    secondary_extension_classes = {".template": TemplateFile}

    base2, ext2 = os.path.splitext(base)

    if ext2 in secondary_extension_classes:
        return secondary_extension_classes[ext2]

    classes = {".html": HTMLFile, ".redirect": RedirectFile, ".md": MarkdownFile}

    if ext in classes:
        return classes[ext]

    return File
